#Setup#####
#Setting working directory
setwd(dirname(rstudioapi::getActiveDocumentContext()$path))
setwd("D:\\P8")
#assigning how many cores to run on
registerDoMC(cores = 4)

#Clear workspace (only used if needed)
rm(list=ls())

#Packages
install.packages("sf", dependencies = TRUE)
install.packages("raster")
install.packages("lattice")
install.packages("caret")
install.packages("mapview")
install.packages("sp")
install.packages("mlr")
install.packages("rattle") #ikke brugt pt-
install.packages("nnet")
install.packages("NeuralNetTools")
install.packages("doMC", repos="http://R-Forge.R-project.org")
install.packages("corrplot")

#Libaries
library(raster)
library(lattice)
library(caret)
library(mapview)
library(sp)
library(rgdal)
library(rpart)
library(mlr)
library(sf)
library(data.table)
library(rattle) #ikke brugt pt
library(nnet)
library(NeuralNetTools)
library(doMC)
library(corrplot)

#
#Data#####
#
#Loading in our data as bricks
sentinel10 <- brick("data/mosaics/10mbands_m.tif")
indices <- brick("data/mosaics/indices_m.tif")
sentinel20 <- brick("data/mosaics/20mbands_c.tif")
#
#Loading our validation data set
training <- read_sf("data/ValidationSet.shp")
#
#Resampling our 20 m bands to 10 m
sentinel20_r <- raster::resample(sentinel20,sentinel10, method = "ngb", filename="20mbands_r", overwrite=TRUE)
#Stacking our data into one stack
allpredictors <- stack(sentinel10,indices,sentinel20_r)
#
#Creating trainingdata#####
#extracting pixels where training data is present on all layers.
extr <- extract(allpredictors, training, df=TRUE)
extr <- merge(extr, training, by.x="ID", by.y="id")

#Giving more meaningfull names to the bands in our extr variable.
setnames(extr, old=c("X10mbands_m.1","X10mbands_m.2","X10mbands_m.3","X10mbands_m.4","X10mbands_m.5","X10mbands_m.6","X10mbands_m.7","X10mbands_m.8","X10mbands_m.9","X10mbands_m.10","X10mbands_m.11","X10mbands_m.12","X10mbands_m.13","X10mbands_m.14","X10mbands_m.15","X10mbands_m.16","indices_m.1","indices_m.2","indices_m.3","indices_m.4","indices_m.5","indices_m.6","indices_m.7","indices_m.8","indices_m.9","indices_m.10","indices_m.11","indices_m.12","indices_m.13","indices_m.14","indices_m.15","indices_m.16","X20mbands_c.1","X20mbands_c.2","X20mbands_c.3","X20mbands_c.4","X20mbands_c.5","X20mbands_c.6","X20mbands_c.7","X20mbands_c.8","X20mbands_c.9","X20mbands_c.10","X20mbands_c.11","X20mbands_c.12","X20mbands_c.13","X20mbands_c.14","X20mbands_c.15","X20mbands_c.16","X20mbands_c.17","X20mbands_c.18","X20mbands_c.19","X20mbands_c.20","X20mbands_c.21","X20mbands_c.22","X20mbands_c.23","X20mbands_c.24"), 
         new=c("SummerB2","SummerB3","SummerB4","SummerB8","AutumnB2","AutumnB3","AutumnB4","AutumnB8","WinterB2","WinterB4","WinterB3","WinterB8","SpringB2","SpringB3","SpringB4","SpringB8","SummerSAVI","SummerTCBrightness","SummerTCGreenness","SummerTCWetness","AutumnSAVI","AutumnTCBrightness","AutumnTCGreenness","AutumnTCWetness","WinterSAVI","WinterTCBrightness","WinterTCGreenness","WinterTCWetness","SpringSAVI","SpringTCBrightness","SpringTCGreenness","SpringTCWetness","SummerB5","SummerB6","SummerB7","SummerB8A","SummerB11","SummerB12","AutumnB5","AutumnB6","AutumnB7","AutumnB8A","AutumnB11","AutumnB12","WinterB5","WinterB6","WinterB7","WinterB8A","WinterB11","WinterB12","SpringB5","SpringB6","SpringB7","SpringB8A","SpringB11","SpringB12"))

#Splitting the training data
trainids <- createDataPartition(extr$class_c,list=FALSE,p=0.7)

#Defining training data and testing data
trainDat <- extr[trainids,]
testDat <- extr[-trainids,]
trainData_copy <-trainDat

#Listing what predicotrs to use
predictors1 <- c("SummerB2","SummerB3","SummerB4","SummerB8","AutumnB2","AutumnB3","AutumnB4","AutumnB8","WinterB2","WinterB4","WinterB3","WinterB8","SpringB2","SpringB3","SpringB4","SpringB8","SummerSAVI","SummerTCBrightness","SummerTCGreenness","SummerTCWetness","AutumnSAVI","AutumnTCBrightness","AutumnTCGreenness","AutumnTCWetness","WinterSAVI","WinterTCBrightness","WinterTCGreenness","WinterTCWetness","SpringSAVI","SpringTCBrightness","SpringTCGreenness","SpringTCWetness","SummerB5","SummerB6","SummerB7","SummerB8A","SummerB11","SummerB12","AutumnB5","AutumnB6","AutumnB7","AutumnB8A","AutumnB11","AutumnB12","WinterB5","WinterB6","WinterB7","WinterB8A","WinterB11","WinterB12","SpringB5","SpringB6","SpringB7","SpringB8A","SpringB11","SpringB12")

#Giving the predictors a more meaning full name, corrosponding with line 50.
names(allpredictors) <- predictors1
#
#Preparing for PCA####
#Cleaning and transforming data for PCA
test_clean <- testDat
test_clean[,c("ID", "path", "Rasterized", "X", "Y", "layer", "Class", "random", "class_c", "geometry")] <- NULL
trainDat_clean <- trainDat
trainDat_clean[,c("ID", "path", "Rasterized", "X", "Y", "layer", "Class", "random", "class_c", "geometry")] <- NULL


#Preprocessing training and test data for PCA
prComp <- prcomp(trainDat_clean, scale = TRUE)
prComp2 <-prcomp(allpredictors, scale=TRUE)

#preparing PCA for test predictions and confusion matrix,
pca_pred_test <- predict(prComp, newdata = test_clean)
pca_pred_test <- as.data.frame(pca_pred_test)
pca_pred_test <- pca_pred_test[,1:10]

#PCA model parameters
train.data<-data.frame(classe = trainDat$class_c, prComp$x)
train.data <- train.data[,1:11]

#PCA stats and plots####
#PCA stats for plots
std_dev <- prComp$sdev
pr_var <- std_dev^2
prop_varex <- pr_var/sum(pr_var)
sum(prop_varex[1:10])

#Curve plot of PCA diminishing returns
plot(cumsum(prop_varex), xlab = "Principal Component",ylab = "Cumulative Proportion of Variance Explained",type = "b")
abline(h=0.975,col='red',v=10)

#Correlation plot
cor_matrix<-abs(cor(trainDat_clean))
diag(cor_matrix)<-0
corrplot(cor_matrix, method="square", tl.cex = 0.6)


#Resuable model parameters####
#Regular parameters
metric <- "Accuracy"
cross_validation <- trainControl(method="cv",search="random", number=10)
Number_of_tune_tries <- 15
train_predictors <- trainDat[,c(predictors1)]
preprocessed <- c("scale", "center")
preprocessed1 <- c("scale", "center", "pca")

#Random Forest#####
#Creating the models
#First model, with no preprocessing
#Creating the RF+PCA model.
model_rf <- caret::train(classe~.,data=train.data, 
                         method="rf", 
                         metric=metric,
                         ntree=1000,
                         tuneLength= Number_of_tune_tries,
                         trControl=cross_validation)


#Making the prediction for the RF+PCA
prediction_PCA <- predict(allpredictors,model_rf, filename="prediction_rf_pca", progress ="window")

#trying the model on the test data.
pred_test <- predict(model_rf, pca_pred_test)
rf_pca_cm <- confusionMatrix(pred_test, as.factor(testDat$class_c))

#
#RF Stats and Plots#####
#plotting a tree from the forest
#Creating a tree model, just to see an example of what a plot could look like.
model_for_plot <- caret::train(classe~.,data=train.data,
                               method="rpart2",
                               trControl=cross_validation)

#plot a tree
fancyRpartPlot(model_for_plot$finalModel)

#NNet#####
# train model with neural network no pca
NNModelPCA <- caret::train(classe~.,data=train.data,
                           method = "nnet",
                           metric=metric,
                           tuneLength= Number_of_tune_tries,
                           trControl=cross_validation,
                           preProcess=preprocessed,
                           na.action = na.omit)

#making the prediction using Nnet+PCA (virker ikke)
NNpredictionPCA <- predict(allpredictors,NNModelPCA, filename="prediction_nn_pca", progress ="window", overwrite=TRUE)

#trying the model on the test data.
nn_pred_test <- predict(NNModelPCA, pca_pred_test)
NN_PCA_cm <- confusionMatrix(nn_pred_test, as.factor(testDat$class_c))
#
#NNet Stats and Plots####
#Nerual Network Plot with pca
plotnet(NNModelPCA$finalModel, y_names = "Kudzu", rel_rsc = 0.7, pos_col = 'lightgreen', neg_col = 'red', circle_col = 'darkseagreen1', bord_col = 'darkseagreen1')
title("Neural Network")


#SVM####
#Creating a SVM-classification model
#seting seeds (required for SVM and KNN)
set.seed(2000)

#building the SVM model with PCA.
SVMModelPCA <- caret::train(classe~.,data=train.data,
                            method = "svmRadial",
                            metric=metric,
                            tuneLength= Number_of_tune_tries,
                            trControl=cross_validation,
                            preProcess=preprocessed,
                            na.action = na.omit)

#making the SVM prediction
SVMprediction <- predict(allpredictors,SVMModelPCA, filename="prediction_SVM", progress ="window", overwrite=TRUE)

#trying the model on the test data.
svm_pred_test <- predict(SVMModelPCA, pca_pred_test)
svm_pca_cm <- confusionMatrix(svm_pred_test, as.factor(testDat$class_c))

#Naive Bayes####
#building the Naive Bayes model with PCA.
NBModelPCA <- caret::train(classe~.,data=train.data,
                            method = "naive_bayes",
                            metric=metric,
                            tuneLength= Number_of_tune_tries,
                            trControl=cross_validation,
                            preProcess=preprocessed,
                            na.action = na.omit)

  #making the SVM prediction
NB_PCA_Pred <- predict.preProcess(allpredictors,NBModelPCA, filename="prediction_NB", progress ="window", overwrite=TRUE)

#trying the model on the test data.
nb_pred_test <- predict(NBModelPCA, pca_pred_test)
NB_pca_cm <- confusionMatrix(nb_pred_test, as.factor(testDat$class_c))

#Boosted Logistic Regression####
#Building the BLR model
BLRModelPCA <- caret::train(classe~.,data=train.data,
                           method = "LogitBoost",
                           metric=metric,
                           tuneLength= Number_of_tune_tries,
                           trControl=cross_validation,
                           preProcess=preprocessed,
                           na.action = na.omit)

#making the SVM prediction
BLR_PCA_Pred <- predict(allpredictors,BLRmodel, filename="prediction_BLR", progress ="window", overwrite=TRUE)

#trying the model on the test data.
BLR_pred_test <- predict(BLRModelPCA, pca_pred_test)
BLR_pca_cm <- confusionMatrix(BLR_pred_test, as.factor(testDat$class_c))
